#ifndef Bildschirm_H
#define Bildschirm_H

#include "Array.h"
#include "Critical.h"
#include "Punkt.h"

//! DirectX 11 Types

struct ID3D11Device;
struct ID3D11DeviceContext;
struct IDXGISwapChain;
struct ID3D11Texture2D;
struct ID3D11SamplerState;
struct ID3D11ShaderResourceView;
struct ID3D11RenderTargetView;
struct ID3D11DepthStencilView;
struct ID3D11DepthStencilState;
struct ID3D11RasterizerState;
struct ID3D11BlendState;
struct D3D11_VIEWPORT;

//! DirectX 9 Types

struct IDirect3D9;
struct IDirect3DDevice9;
struct IDirect3DSurface9;
struct _D3DLOCKED_RECT;

namespace Framework
{
    class Bild;              //! Bild.h
    class WFenster;          //! Fenster.h
    class Bildschirm;        //! aus dieser Datei
    class Zeichnung;         //! Zeichnung.h
    class Zeichnung3D;       //! Zeichnung3D.h
    class ZeitMesser;        //! Zeit.h
    struct MausEreignis;     //! MausEreignis.h
    struct TastaturEreignis; //! TastaturEreignis.h
    class ToolTip;           //! Tooltip.h
    class PixelShader;       //! Shader.h
    class VertexShader;      //! Shader.h
    class Kam3D;             //! Kam3D.h
    class Render3D;          //! Render3D.h
    class DXVertexBuffer;    //! DXBuffer.h
    class DXIndexBuffer;     //! DXBuffer.h
    class Textur;            //! Textur.h
    class TexturModel;       //! TexturModel.h
    class GraphicsApi;       //! GraphicsApi.h
    class DirectX9;          //! GraphicsApi.h

    enum GraphicApiType
    {
        NOT_SUPPORTED, //! no graphic api supported
        DIRECTX9,      //! only 2d
        DIRECTX11,     //! 3d simple phong model without shadows
        DIRECTX12      //! 3d phong model with raytraycing
    };

    //! Eine Struktur, die Informationen ber einen Monitor beinhaltet
    struct Monitor
    {
        int x, y, breite, height; //! Koordinaten des Monitors und die Auflsung
        bool existiert; //! Speichert, ob der Monitor wirklich existiert
        Text name;
    };

    //! Diese Klasse verwaltet das Bild auf dem Bildschirm
    class Bildschirm : public virtual ReferenceCounter
    {
    protected:
        GraphicsApi* api;
        WFenster* fenster;
        RCArray<Zeichnung>* members;
        int fillColor;
        int deckFarbe;
        Zeichnung* onTop;
        bool renderOnTop;
        bool renderZeichnungen;
        bool rendering;
        ZeitMesser* renderZeit;
        Critical cs;
        RCArray<ToolTip>* tips;
        bool testRend;
        bool fill;
        bool rend;
        std::mutex mutex;
        int waitingUserInput;
        std::condition_variable waiting;

    public:
        //! Konstruktor
        //! \param fenster Das Fenster, dessen Inhalt verwaltet werden soll
        DLLEXPORT Bildschirm(WFenster* fenster);
        //! Destruktor
        DLLEXPORT virtual ~Bildschirm();
        //! Dies ist notwendig, falls mehrere Threads gleichzeitig den
        //! Bildschirm benutzen. Wenn lock() von zwei threads aufgerufen wird,
        //! wartet der letzte so lange, bis der erste unlock() aufgerufen hat.
        DLLEXPORT virtual void lock();
        //! Dies ist notwendig, falls mehrere Threads gleichzeitig den
        //! Bildschirm benutzen. Wenn lock() von zwei threads aufgerufen wird,
        //! wartet der letzte so lange, bis der erste unlock() aufgerufen hat.
        DLLEXPORT virtual void unlock();
        //! Legt fest, ob der Bildschirm nach jedem Bild neu mit einer Farbe
        //! ausgefllt wird (Standartmig gesetzt) \param f 1, falls das Bild
        //! vor dem Zeichnen zurckgesetzt werden soll
        DLLEXPORT virtual void setFill(bool f);
        //! Aktualisiert die Objekte, mit der die Grafikkarte verwaltet wird
        DLLEXPORT virtual void update();
        //! Legt fest, ob von dem Zeichnen berprft wird, ob sich das Bild vom
        //! letzten unterscheidet (Standartmig gesetzt) \param tr 1, wenn nur
        //! gezeichnet werden soll, wenn es notwendig ist
        DLLEXPORT virtual void setTestRend(bool tr);
        //! Legt fest, ob die 2D GUI Zeichnungen gezeichnet werden sollen
        //! (Standartmig gesetzt) \param rO 1, falls die Zeichnungen
        //! gezeichnet werden sollen
        DLLEXPORT virtual void setRenderZeichnungen(bool rO);
        //! Legt fest, ob eine Zeichnung ber allen anderen Zeichnungen
        //! gezeichnet werden soll (Standartmig nicht gesetzt) \param onTop 1,
        //! falls eine Zeichnung ber allem gezeichnet werden soll
        DLLEXPORT virtual void setOnTop(bool onTop);
        //! Setzt die Zecihnung, die ber allem gezeichnet werden soll, falls
        //! setOnTop( 1 ) gesetzt wurde \param obj Die Zeichnung
        DLLEXPORT virtual void setOnTopZeichnung(Zeichnung* obj);
        //! Setzt eine Farbe, die nach den Zeichnungen mit alphablending ber
        //! das gesamte Bild geblendet wird, wenn setOnTop( 1 ) gesetzt wurde.
        //! Die zeichnung, die mit setOnTopZeichnung() gesetzt wurde ist davon
        //! nicht betroffen \param f Die Farbe im A8R8G8B8 Format
        DLLEXPORT virtual void setdeckFarbe(int f);
        //! Fgt dem Bild eine Zeichnung hinzu, die immer gezeichnet wird, wenn
        //! setRenderZeichnungen( 1 ) gesetzt wurde. \param obj Die Zeichnung
        DLLEXPORT virtual void addMember(Zeichnung* obj);
        //! Entfernt eine Zeichnung aus dem Bild. Darf nicht aufgerufen werden,
        //! wrend doMausEreignis(), doTastaturEreignis(), tick() oder render()
        //! aufgerufen wird. \param obj Die Zeichnung (ohne erhhten reference
        //! Counter)
        DLLEXPORT virtual void removeMember(Zeichnung* zObj);
        //! Zeichnet ein Bild und prsentiert es auf dem Bildschirm
        DLLEXPORT virtual void render();
        //! Setzt die Farbe, mit der das Bild for dem Zeichnen gefllt wird,
        //! wenn setFll( 1 ) gesetzt wurde \param f Die Farbe im A8R8G8B8
        //! Format
        DLLEXPORT virtual void setFillFarbe(int f);
        //! Legt fest, ob das Bild im Vollbildmodus prsentiert werden soll.
        //! (Standartmig nicht gesetzt) \param fullscreen 1, fr Vollbildmodus
        DLLEXPORT virtual void setVollbild(bool fullscreen);
        //! Verarbeitet die Zeit, die seit dem letzten Aufruf der Funktion
        //! vergangen ist Ruft die tick Funktionen aller sich im Bild
        //! befindenden Zeichnungen auf \param tickval Die vergangene Zeit in
        //! Sekunden
        DLLEXPORT virtual void tick(double tickval);
        //! Setzt die Auflsung des Bildes, das angezeigt wird. Es wird von der
        //! Grafikkarte automatisch skalliert, so das es das Fenster ausfllt
        //! \param breite Die Breite des Bildes in Pixeln
        //! \param height Die Hhe des Bildes in Pixeln
        DLLEXPORT virtual void setBackBufferSize(int breite, int height);
        //! Setzt die Auflsung des Bildes, das angezeigt wird. Es wird von der
        //! Grafikkarte automatisch skalliert, so das es das Fenster ausfllt
        //! \param size Die Breite und Hhe in Pixeln
        DLLEXPORT virtual void setBackBufferSize(Punkt& size);
        //! Verarbeitet ein Maus Ereignis. Wird vom Framework automatisch
        //! aufgerufen. Gibt das Ereignis automatisch an alle sich im Bild
        //! befindenden Zeichnungen weiter \param me Das Ereignis
        DLLEXPORT virtual void doMausEreignis(MausEreignis& me);
        //! Verarbeitet ein Tastatur Ereignis. Wird vom Framework automatisch
        //! aufgerufen Gibt das Ereignis automatisch an alle sich im Bild
        //! befindenden Zeichnungen weiter \param te Das Ereignis
        DLLEXPORT virtual void doTastaturEreignis(TastaturEreignis& te);
        //! Fgt einen Tooltip hinzu.
        //! \param tip Der Tooltip
        DLLEXPORT virtual void addToolTip(ToolTip* tip);
        //! Gibt das Bild zurck, in das die 2D GUI Zeichnungen des Frameworks
        //! gezeichnet werden
        DLLEXPORT virtual Bild* getRenderBild() const;
        //! Gibt das Bild ohne erhhten Reference Counter zurck, in das die 2D
        //! GUI Zeichnungen des Frameworks gezeichnet werden
        DLLEXPORT virtual Bild* zRenderBild() const;
        //! Gibt ein Array von 2D GUI Zeichnungen zurck, die sich im Bild
        //! befinden
        DLLEXPORT virtual Iterator<Zeichnung*> getMembers() const;
        //! Gibt die Farbe im A8R8G8B8 Format zurck, mit der das Bild vor dem
        //! Zeichnen befllt wird
        DLLEXPORT virtual int getFillFarbe() const;
        //! Gibt zurck, ob man sich im Vollbildmodus befindet
        DLLEXPORT virtual bool istVolbild() const;
        //! Gibt die Auflsung in Pixeln zurck, in der gezeichnet wird
        DLLEXPORT virtual const Punkt getBackBufferSize() const;
        //! Wartet, bis das Zeichnen des aktuellen Bildes abgeschlossen ist
        DLLEXPORT virtual void warteAufRendern() const;
        //! Gibt die Zeit in Sekunden zurck, die bentigt wurde, um das letzte
        //! Bild zu zeichnen
        DLLEXPORT virtual double getRenderZeit() const;
        //! Gibt die Grafik API zurck (ohne erhhten Reference Counter)
        DLLEXPORT GraphicsApi* zGraphicsApi() const;
        //! Gibt die Grafik API zurck
        DLLEXPORT GraphicsApi* getGraphicsApi() const;
    };
#ifdef WIN32
    //! Diese Klasse verwaltet das Bild auf dem Bildschirm ohne 3D Elemente
    class Bildschirm2D : public Bildschirm
    {
    public:
        //! Konstruktor
        //! \param fenster Das Fenster, dessen Inhalt verwaltet werden soll
        DLLEXPORT Bildschirm2D(WFenster* fenster);
        //! Destruktor
        DLLEXPORT virtual ~Bildschirm2D();
        //! Zeichnet ein Bild und prsentiert es auf dem Bildschirm
        DLLEXPORT virtual void render();
    };

    //! Diese Klasse verwaltet das Bild auf dem Bildschirm mit 3D Elementen
    class Bildschirm3D : public Bildschirm
    {
    private:
        RCArray<Kam3D>* kameras;
        bool rend3D;

    public:
        //! Konstruktor
        //! \param fenster Das Fenster, dessen Inhalt verwaltet werden soll
        DLLEXPORT Bildschirm3D(WFenster* fenster);
        DLLEXPORT Bildschirm3D(WFenster* fenster, GraphicApiType apiTyp);
        DLLEXPORT Bildschirm3D(WFenster* fenster, GraphicsApi* api);
        //! Destruktor
        DLLEXPORT virtual ~Bildschirm3D();
        //! Fgt dem Bildschirm eine Kamera hinzu
        //! \param obj Die Kamera
        DLLEXPORT void addKamera(Kam3D* obj);
        //! Entfernt eine kamera von dem Bildschirm
        DLLEXPORT void removeKamera(Kam3D* zObj);
        //! Verarbeitet die Zeit, die seit dem letzten Aufruf der Funktion
        //! vergangen ist Ruft die tick Funktionen aller sich im Bild
        //! befindenden Zeichnungen und der Kameras auf \param tickval Die
        //! vergangene Zeit in Sekunden
        DLLEXPORT void tick(double tickval);
        //! Verarbeitet ein Maus Ereignis. Wird vom Framework automatisch
        //! aufgerufen. Gibt das Ereignis automatisch an alle sich im Bild
        //! befindenden Zeichnungen und Kameras weiter \param me Das Ereignis
        DLLEXPORT void doMausEreignis(MausEreignis& me);
        //! Verarbeitet ein Tastatur Ereignis. Wird vom Framework automatisch
        //! aufgerufen Gibt das Ereignis automatisch an alle sich im Bild
        //! befindenden Zeichnungen und Kameras weiter \param te Das Ereignis
        DLLEXPORT void doTastaturEreignis(TastaturEreignis& te);
        //! Zeichnet ein Bild und prsentiert es auf dem Bildschirm
        DLLEXPORT void render();
    };

    //! Findet die Psition und Auflsung eines Monitors heraus
    //! \param id Die ID des Monitors. Falls der Monitor nicht gefunden wurde
    //! ist der existiert flag der zurckgegebenen Monitor Struktur 0
    DLLEXPORT Monitor getMonitor(int id);
#endif
} // namespace Framework

#endif